#ifndef ROSTERINDEX_H
#define ROSTERINDEX_H

#include <QMap>
#include <QHash>
#include <QMultiHash>
#include <definitions/rosterindextyperole.h>
#include <interfaces/irostersmodel.h>

class RosterIndex :
			public QObject,
			public IRosterIndex
{
	Q_OBJECT;
	Q_INTERFACES(IRosterIndex);
public:
	RosterIndex(int AType, const QString &AId);
	~RosterIndex();
	QObject *instance() { return this; }
	//IRosterIndex
	virtual int type() const { return data(RDR_TYPE).toInt(); }
	virtual QString id() const { return data(RDR_INDEX_ID).toString(); }
	virtual IRosterIndex *parentIndex() const { return FParentIndex; }
	virtual void setParentIndex(IRosterIndex *AIndex);
	virtual int row() const;
	virtual void appendChild(IRosterIndex *AIndex);
	virtual IRosterIndex *child(int ARow) const { return FChilds.value(ARow,0); }
	virtual int childRow(const IRosterIndex *AIndex) const;
	virtual int childCount() const { return FChilds.count(); }
	virtual bool removeChild(IRosterIndex *AIndex);
	virtual void removeAllChilds();
	virtual Qt::ItemFlags flags() const { return FFlags; }
	virtual void setFlags(const Qt::ItemFlags &AFlags) { FFlags = AFlags; }
	virtual void insertDataHolder(IRosterDataHolder *ADataHolder);
	virtual void removeDataHolder(IRosterDataHolder *ADataHolder);
	virtual QVariant data(int ARole) const;
	virtual QMap<int,QVariant> data() const;
	virtual void setData(int ARole, const QVariant &AData);
	virtual QList<IRosterIndex *> findChild(const QMultiHash<int, QVariant> AData, bool ASearchInChilds = false) const;
	virtual bool removeOnLastChildRemoved() const { return FRemoveOnLastChildRemoved; }
	virtual void setRemoveOnLastChildRemoved(bool ARemove) { FRemoveOnLastChildRemoved = ARemove; }
	virtual bool removeChildsOnRemoved() const { return FRemoveChildsOnRemoved; }
	virtual void setRemoveChildsOnRemoved(bool ARemove) { FRemoveChildsOnRemoved = ARemove; }
	virtual bool destroyOnParentRemoved() const { return FDestroyOnParentRemoved; }
	virtual void setDestroyOnParentRemoved(bool ADestroy) {FDestroyOnParentRemoved = ADestroy; }
signals:
	void dataChanged(IRosterIndex *AIndex, int ARole);
	void dataHolderInserted(IRosterDataHolder *ADataHolder);
	void dataHolderRemoved(IRosterDataHolder *ADataHolder);
	void childAboutToBeInserted(IRosterIndex *AIndex);
	void childInserted(IRosterIndex *AIndex);
	void childAboutToBeRemoved(IRosterIndex *AIndex);
	void childRemoved(IRosterIndex *AIndex);
	void indexDestroyed(IRosterIndex *AIndex);
protected slots:
	void onDataHolderChanged(IRosterIndex *AIndex, int ARole);
	void onRemoveByLastChildRemoved();
	void onDestroyByParentRemoved();
private:
	bool FBlokedSetParentIndex;
	bool FRemoveChildsOnRemoved;
	bool FDestroyOnParentRemoved;
	bool FRemoveOnLastChildRemoved;
	Qt::ItemFlags FFlags;
	IRosterIndex *FParentIndex;
	QList<IRosterIndex *> FChilds;
	QMap<int, QVariant> FData;
	QHash<int, QMultiMap<int,IRosterDataHolder *> > FDataHolders;
};

#endif // ROSTERINDEX_H
